@@ -19,24 +19,6 @@
 
 #   Routines for toeplitz SSA
 
-tcirc.old <- function(F, L = (N + 1) %/% 2) {
-  N <- length(F)
-
-  f <- fft(c(F, rep(0, L-1)))
-  R <- fft(f * Conj(f), inverse = TRUE)[1:L] / (N+L-1) / seq(from = N, to = N-L+1, by = -1)
-
-  .res <- list()
-  .res$C <- as.vector(fft(c(R, rev(R[-1]))))
-  .res$L <- L
-  return (.res)
-}
-
-tmatmul.old <- function(C, v) {
-  v <- as.vector(fft(C$C * fft(c(v, rep(0, C$L-1))), inverse = TRUE))
-  Re((v/length(C$C))[1:C$L])
-}
-
-
 Lcor <- function(F, L) {
   storage.mode(F) <- "double"
   storage.mode(L) <- "integer"
@@ -119,7 +101,9 @@ decompose.toeplitz.ssa.nutrlan <- function(x,
   x
 }
 
-decompose.toeplitz.ssa.eigen <- function(x, ...,
+decompose.toeplitz.ssa.eigen <- function(x,
+                                         neig = min(50, L, K),
+                                         ...,
                                          force.continue = FALSE) {
   N <- x$length; L <- x$window; K <- N - L + 1
 
@@ -132,14 +116,10 @@ decompose.toeplitz.ssa.eigen <- function(x, ...,
   h <- .get.or.create.hmat(x)
 
   # Do decomposition
-  if ("neig" %in% names(list(...)))
-    warning("'neig' option ignored for SSA method 'eigen', computing EVERYTHING",
-            immediate. = TRUE)
-
   C <- toeplitz(Lcor(F, L))
   S <- eigen(C, symmetric = TRUE)
 
-  .set(x, "U", S$vectors)
+  .set(x, "U", S$vectors[, 1:neig])
 
   lambda <- numeric(L)
   V <- matrix(nrow = K, ncol = L)
@@ -150,14 +130,14 @@ decompose.toeplitz.ssa.eigen <- function(x, ...,
   }
 
   # Save results
-  .set(x, "lambda", lambda)
-  .set(x, "V", V)
+  .set(x, "lambda", lambda[1:neig])
+  .set(x, "V", V[, 1:neig, drop = FALSE])
 
   x
 }
 
 decompose.toeplitz.ssa.svd <- function(x,
-                                       neig = min(L, K),
+                                       neig = min(50, L, K),
                                        ...,
                                        force.continue = FALSE) {
   N <- x$length; L <- x$window; K <- N - L + 1
@@ -240,5 +220,5 @@ decompose.toeplitz.ssa <- function(x,
   stop("Unsupported SVD method for Toeplitz SSA!")
 }
 
-calc.v.toeplitz.ssa <- function(x, idx, env = .GlobalEnv, ...)
+calc.v.toeplitz.ssa <- function(x, idx, ...)
   x$V[, idx]
